import React, { useEffect, useState } from "react";

import IconButton from "@mui/material/IconButton";
import ChevronRight from "@mui/icons-material/ChevronRight";

import { signUp } from "../api/auth";
import Close from "@mui/icons-material/Close";
import Card from "@mui/material/Card";
import Grid from "@mui/material/Grid";
import CardActions from "@mui/material/CardActions";
import CardContent from "@mui/material/CardContent";
import CardMedia from "@mui/material/CardMedia";
import Button from "@mui/material/Button";
import Typography from "@mui/material/Typography";
import { useNavigate } from "react-router-dom";
import isEmpty from "validator/lib/isEmpty";
import errorAlert from "../Helpers/errorAlert";
import successAlert from "../Helpers/successAlert";
import { purchaseTicket } from '../api/purchases';
import { purple } from "@mui/material/colors";

// Event card - Using material UI framework to bootstrap this process
// Event Data - Individual event data
// Type - small or large
// SetSelectedEvent - Set event to be shown in modal

const EventCard = ({ eventData, type, setSelectedEvent, user }) => {
  let navigate = useNavigate();

  //Show/Hide ticket purchase component boolean
  const [showBuyTicket, setShowBuyTicket] = useState(false);

  //Current purchase stage
  //1 = Ticket info
  //2 = Contact Information
  //3 = Payment information
  const [stage, setStage] = useState(1);

  //Contact info form
  const [contactFormData, setContactFormData] = useState({
    addressLine1: "1 Melville Road",
    addressLine2: "",
    city: "Devon",
    county: "Devon",
    postCode: "DV12 5TR",
    cardholderName: "Fin Wallis",
    cardNumber: 1111111111111111,
    cardExpiryDate: "12/24",
    cardCVC: 222,
    errorMsg: false,
  });

  // Validate postcode
  function isPostcode(postcode) {
    postcode = postcode.replace(/\s/g, "");
    var regex = /^[A-Z]{1,2}[0-9]{1,2}[A-Z]{0,1} ?[0-9][A-Z]{2}$/i;
    return regex.test(postcode);
  }

  // When form values change on stage 2
  const handleContactFormChange = (event) => {
    setContactFormData({
      ...contactFormData,
      [event.target.name]: event.target.value,
      errorMsg: "",
    });
    console.log(contactFormData);
  };

  // When user clicks next on stage 2
  const handleContactSubmit = (event) => {
    event.preventDefault();
    console.log(contactFormData);

    // Form validation
    if (
      isEmpty(contactFormData.addressLine1) ||
      isEmpty(contactFormData.city) ||
      isEmpty(contactFormData.county) ||
      isEmpty(contactFormData.postCode)
    ) {
      setContactFormData({
        ...contactFormData,
        errorMsg: "All fields are required except address line 2",
      });
    } else if (!isPostcode(contactFormData.postCode)) {
      setContactFormData({
        ...contactFormData,
        errorMsg: "Invalid UK postcode",
      });
    } else {
      const {
        addressLine1,
        addressLine2,
        city,
        county,
        postCode,
        cardholderName,
        cardNumber,
        cardExpiryDate,
        cardCVC,
      } = contactFormData;
      console.log(user);

      const data = {addressLine1, addressLine2, city, county, postCode, cardholderName, cardNumber, cardExpiryDate, cardCVC, event_id: eventData._id, amount: eventData.ticketPrice, user_id: user._id}

      console.log(data);

      purchaseTicket(data)
        .then((response) => {
          console.log("axios success", response);
          setContactFormData({
            addressLine1: "",
            addressLine2: "",
            city: "",
            county: "",
            postCode: "",
            cardholderName: "",
            cardNumber: "",
            cardExpiryDate: "",
            cardCVC: "",
            successMsg: response.data.successMessage,
          });
        })
        .catch((err) => {
          console.log("Axios error:", err);
          setContactFormData({
            ...contactFormData,
            errorMsg: err.response.data.errorMessage,
          });
        });
    }
  };

  return (
    <>
      {/* Small marketplace card */}
      {type === "small" && (
        <Card>
          <CardMedia
            component="img"
            sx={{ maxHeight: 200 }}
            height={200}
            image={eventData.eventImage}
            alt="Event Image"
          />
          <CardContent>
            <div style={{ overflow: "hidden", textOverflow: "ellipsis" }}>
              <Typography gutterBottom variant="h5" noWrap component="div">
                {eventData.eventTitle}
              </Typography>
            </div>
            {/* Adding text elipsis to add ... automatically */}
            <div
              style={{
                overflow: "hidden",
                textOverflow: "ellipsis",
                height: 70,
              }}
            >
              <Typography variant="body2" color="text.secondary">
                {eventData.eventDescription}
              </Typography>
            </div>
          </CardContent>
          <CardActions>
            <Button
              onClick={() => {
                setSelectedEvent(eventData);
              }}
              sx={{
                backgroundColor: "#9f7aea",
                color: "white",
                padding: "15px",
                marginTop: "40px",
                width: "100%",
              }}
              size="small"
            >
              Book event
            </Button>
          </CardActions>
        </Card>
      )}
      {/* Large marketplace using paper instead */}
      {type === "large" && (
        <Grid container xs={12} spacing={2}>
          {/* Close button */}
          <Grid container xs={12} justifyContent={"flex-end"}>
            <IconButton
              aria-label="Close modal"
              component="span"
              onClick={() => setSelectedEvent(null)}
            >
              <Close />
            </IconButton>
          </Grid>

          {/* Left image panel */}
          <Grid container item xs={6} justifyContent={"center"}>
            <Grid xs={10}>
              <CardMedia
                component="img"
                sx={{ height: 500 }}
                image={eventData.eventImage}
                alt="Event Image"
              />
            </Grid>
          </Grid>

          {/* If user has not clicked 'Buy ticket' show information */}
          <>
            {/* Right info panel */}
            <Grid item container xs={6} spacing={2}>
              <Typography
                variant={!showBuyTicket ? "h3" : "h4"}
                fontWeight={"bolder"}
              >
                {eventData.eventTitle}
              </Typography>
              {!showBuyTicket ? (
                <>
                  {/* Split up the info using MUI Grid  */}
                  <Grid xs={6} item sx={{ marginTop: "25px" }}>
                    <Typography variant={"h6"} fontWeight={"bold"}>
                      Date
                    </Typography>
                    <Typography variant={"body2"}>
                      {eventData.startDate}
                    </Typography>
                  </Grid>
                  <Grid xs={6} item sx={{ marginTop: "25px" }}>
                    <Typography variant={"h6"} fontWeight={"bold"}>
                      Venue
                    </Typography>
                    <Typography item variant={"body2"}>
                      {eventData.venueAddress}
                    </Typography>
                  </Grid>
                  <Grid xs={6} item sx={{ marginTop: "25px" }}>
                    <Typography variant={"h6"} fontWeight={"bold"}>
                      Event Type
                    </Typography>
                    <Typography variant={"body2"}>
                      {eventData.eventGenre}
                    </Typography>
                  </Grid>
                  <Grid xs={6} item sx={{ marginTop: "25px" }}>
                    <Typography variant={"h6"} fontWeight={"bold"}>
                      Time
                    </Typography>
                    <Typography variant={"body2"}>
                      {eventData.startTime} - {eventData.endTime}
                    </Typography>
                  </Grid>
                  {/* Description */}
                  <Typography
                    variant={"h6"}
                    fontWeight={"bold"}
                    sx={{ marginTop: "25px" }}
                  >
                    Description
                  </Typography>
                  <Typography variant={"body2"}>
                    {eventData.eventDescription}
                  </Typography>

                  <Grid xs={6} item sx={{ marginTop: "25px" }}>
                    <div style={{ color: "#9f7aea" }}>
                      <Typography
                        variant={"h6"}
                        fontWeight={"bold"}
                        color={"inherit"}
                      >
                        Current Price
                      </Typography>
                    </div>
                    <Typography variant={"body2"}>
                      £{eventData.ticketPrice}
                    </Typography>
                  </Grid>
                  <Grid xs={6} container item sx={{ marginTop: "25px" }}>
                    {/* Either show login to purchase, redirect to login or to buy ticket */}

                    <Button
                      onClick={() => {
                        //If user is not null, open purchase card, else redirect to login
                        if (user != null) {
                          setShowBuyTicket(!showBuyTicket);
                        } else {
                          navigate("/Login");
                        }
                      }}
                      size="small"
                      sx={{
                        backgroundColor: "#9f7aea",
                        color: "white",
                        padding: "10px",
                      }}
                    >
                      {user !== null ? "Buy Ticket" : "Login to buy"}
                    </Button>
                  </Grid>
                </>
              ) : (
                <>
                  {stage === 1 && (
                    <Grid
                      container
                      sx={{
                        backgroundColor: "#9f7aea",
                        borderRadius: "8px",
                        padding: "15px",
                      }}
                    >
                      <Typography variant={"h6"}>
                        {eventData.eventTitle}
                      </Typography>
                      <Grid
                        item
                        container
                        xs={12}
                        justifyContent={"flex-start"}
                      >
                        <Typography
                          variant={"body2"}
                          sx={{ fontWeight: "normal" }}
                        >
                          Venue:
                        </Typography>
                        <Typography variant={"body2"}>
                          {eventData.venueName}
                        </Typography>
                      </Grid>
                      <Grid item container xs={4} justifyContent={"flex-start"}>
                        <Typography variant={"body2"}>
                          {eventData.startDate}
                        </Typography>
                      </Grid>
                      <Grid item container xs={4} justifyContent={"flex-start"}>
                        <Typography variant={"body2"}>
                          {eventData.startTime} - {eventData.endTime}
                        </Typography>
                      </Grid>
                    </Grid>
                  )}

                  {stage === 2 && (
                    <form
                      className="signup-form space-y-5 px-100"
                      onSubmit={handleContactSubmit}
                      noValidate
                    >
                      {contactFormData.errorMsg &&
                        errorAlert(contactFormData.errorMsg)}
                      {contactFormData.successMsg &&
                        successAlert(contactFormData.successMsg)}
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="addressLine1"
                        value={contactFormData.addressLine1}
                        type="text"
                        placeholder="Address Line 1*"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="addressLine2"
                        value={contactFormData.addressLine2}
                        type="text"
                        placeholder="Address Line 2"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="city"
                        value={contactFormData.city}
                        type="text"
                        placeholder="City"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="county"
                        value={contactFormData.county}
                        type="text"
                        placeholder="County"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="postCode"
                        value={contactFormData.postCode}
                        type="text"
                        placeholder="Postcode"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="cardholderName"
                        value={contactFormData.cardholderName}
                        type="text"
                        placeholder="City"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="cardNumber"
                        value={contactFormData.cardNumber}
                        type="number"
                        minLength={16}
                        maxLength={16}
                        placeholder="County"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="cardExpiryDate"
                        value={contactFormData.cardExpiryDate}
                        type="string"
                        placeholder="Expiration Date"
                        onChange={handleContactFormChange}
                      ></input>
                      <input
                        className="font-montserrat font-medium rounded-full w-full py-4 px-7 shadow-lg"
                        name="cardCVC"
                        value={contactFormData.cardCVC}
                        type="number"
                        minLength={3}
                        maxLength={3}
                        placeholder="CVC Number"
                        onChange={handleContactFormChange}
                      ></input>
                      <Grid
                        container
                        item
                        justifyContent={"space-between"}
                        sx={{ height: "50px" }}
                      >
                        <IconButton
                          onPress={() => {
                            setStage(stage - 1);
                          }}
                        >
                          Back
                        </IconButton>

                        <button
                          className="py-4 px-14 text-white bg-purple-500 hover:bg-purple-400 font-montserrat font-medium rounded-full"
                          type="submit"
                        >
                          Buy ticket for £{eventData.ticketPrice}
                        </button>
                      </Grid>
                    </form>
                  )}

                  {stage === 1 && (
                    <Grid
                      container
                      item
                      justifyContent={"space-between"}
                      sx={{ height: "50px" }}
                    >
                      <IconButton
                        onPress={() => {
                          setShowBuyTicket(false);
                        }}
                      >
                        Cancel
                      </IconButton>

                      <IconButton
                        onClick={() => {
                          setStage(stage + 1);
                          console.log(stage);
                        }}
                      >
                        Next
                        <ChevronRight />
                      </IconButton>
                    </Grid>
                  )}
                </>
              )}
            </Grid>
          </>
        </Grid>
      )}
    </>
  );
};

export default EventCard;
